//= require <dSS/data/ModulatorStore>
//= require <dSS/data/DeviceStore>
//= require <dSS/data/ZoneStore>
//= require <dSS/data/GroupStore>
//= require <dSS/data/SceneStore>

Ext.namespace('dSS', 'dSS.data');

dSS.data.LoaderImpl = Ext.extend(Ext.util.Observable, {

    constructor: function() {
        this.modulatorStore = new dSS.data.ModulatorStore({
            url: '/json/apartment/getCircuits'
        });
        this.deviceStore = new dSS.data.DeviceStore();
        this.zoneStore = new dSS.data.ZoneStore();
        this.sceneStore = new dSS.data.SceneStore();
        this.groupStore = new dSS.data.GroupStore();

        this.modulatorStore.on('loadcomplete',
                               this.loadStructure.createDelegate(this));

        dSS.MessageBus.on('reloadapartment', this.load.createDelegate(this),
                           this);
    },

    load: function() {
        if (this.modulatorStore !== undefined) {
            this.modulatorStore.load();
        } else {
            dSS.MessageBus.fireEvent('structureloaded');
        }
    },

    getModulatorStore: function() {
        return this.modulatorStore;
    },

    getDeviceStore: function() {
        return this.deviceStore;
    },

    getZoneStore: function() {
        return this.zoneStore;
    },

    getSceneStore: function() {
        return this.sceneStore;
    },

    getGroupStore: function() {
        return this.groupStore;
    },

    loadStructure: function() {
        Ext.Ajax.request({
            url: '/json/apartment/getStructure',
            disableCaching: true,
            timeout: 300000,
            method: "GET",
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    this.processStructure(jsonData.result);
                    dSS.MessageBus.fireEvent('structureloaded');
                }
                catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not load apartment structure") + ": \"" + err + "\"");
                    dSS.MessageBus.fireEvent('structureloaded');
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not load") + ": \"" + request.url + "\"");
                dSS.MessageBus.fireEvent('structureloaded');
            }
        });
    },

    processStructure: function(structure) {
        var devices = [];
        var zones = [];
        var groups = [];
        var zoneStore = this.zoneStore;
        var modulatorStore = this.modulatorStore;
        var deviceStore = this.deviceStore;
        var groupStore = this.groupStore;

        Ext.each(structure.apartment.zones, function(zone) {
            if (zone.id === 0) {
                zone.name = zone.name ? zone.name: dSS._("All devices");
                Ext.each(zone.devices, function(device) {
                    device.firstSeen = Date.parseDate(device.firstSeen,
                                                      "Y-m-d H:i:s", true);
                    device.inactiveSince = Date.parseDate(
                                                device.inactiveSince,
                                                "Y-m-d H:i:s", true);
                    devices.push(device);
                });

                groups.push({
                    id: 0,
                    sortorder: 0,
                    color: -1,
                    name: dSS._("All devices")
                });

                for (var z = 0; z < zone.groups.length; z++) {
                    var grp = zone.groups[z];

                    var order = 200;
                    var cssTune = "";
                    if ((grp.id !== 0) && (grp.isValid === true) &&
                        (((!dSS.util.isUserGroup(grp.id) &&
                          (grp.devices.length > 0)) ||
                          dSS.util.isUserGroup(grp.id)))) {
                        var name = grp.name ? grp.name: dSS._("Group") +
                                   ' #' + grp.id;
                        if (!dSS.util.isUserGroup(grp.id)) {
                            name = dSS._(dSS.config.groupNames[grp.id]);
                            order = grp.id;
                            cssTune = "Small";
                        } else {
                            cssTune = "UserGroup";
                        }

                        groups.push({
                            id: grp.id,
                            color: grp.color,
                            sortorder: order,
                            css: dSS.config.GroupColorIconCls[grp.color] +
                                 cssTune,
                            name: name
                        });
                    }
                }
                groups.push({
                    id: -1,
                    sortorder: 199,
                    color: -1,
                    name: '&mdash;&mdash;&mdash;&mdash;&mdash;&mdash;' +
                          '&mdash;&mdash;&mdash;&mdash;&mdash;'
                });
            }

            var unnamedInZone = false;
            Ext.each(zone.devices, function(device) {
                if ((device.name === undefined) || (device.name.length === 0)) {
                    unnamedInZone = true;
                }
                var i;
                for (i = 0; i < devices.length; i++) {
                    if (devices[i].dSUID === device.dSUID) {
                        devices[i].zoneId = zone.id;
                        devices[i].zoneName = zone.name;
                    }
                }
            });

            zones.push({
                id: zone.id,
                unnamedDevices: unnamedInZone,
                alternativeSort: zone.id === 0 ? 1 : 0,
                name: zone.name ? zone.name: dSS._("Zone") + ' #' + zone.id,
                primary: ('firstZoneOnDSMeter' in zone) ? true: false
            });

            zoneStore.loadData({
                zones: zones
            });
        });
        Ext.each(devices, function(device) {
            var index = modulatorStore.find('dsuid',
                        new RegExp("^" + device.meterDSUID + "$"));
            if (index > - 1) {
                var modulator = modulatorStore.getAt(index);
                device.meterName = modulator.get("name");
            }
        });
        groupStore.loadData({
            groups: groups
        });

        deviceStore.loadData({
            devices: devices
        });
        modulatorStore.each(
            function(rec) {
                rec.set('numDevices',
                        deviceStore.queryBy(function(record, id) {
                            if (record.get('meterDSUID') == rec.get('dsuid')) {
                                return true;
                            }
                            return false;
                        }).length);
            });
        modulatorStore.commitChanges();
        this.sceneStore.loadScenes();
    }
});

